@@ -18,6 +18,7 @@ except ImportError:
     sys.stderr.write("Compiled fill_sinks not available. Falling back to "
                      "Python version\n")
     from .raster import fill_sinks
+
 try:
     from scipy.interpolate import griddata
     HAS_SCIPY = True
@@ -82,7 +83,7 @@ class Grid(object):
 
 class RegularGrid(Grid):
     """ Regular (structured) grid class. A RegularGrid contains a fixed number
-    of rows and columns with a regular spacing and a scalar or vector field
+    of rows and columns with a constant spacing and a scalar or vector field
     defined as `Z`.
     """
     def __init__(self, hdr, Z=None):
@@ -351,13 +352,22 @@ class RegularGrid(Grid):
         if method == "nearest":
             return self.sample_nearest(x, y)
         elif method == "linear":
-            import scipy.interpolate
-            Xd, Yd = self.center_coords()
-            return scipy.interpolate.griddata((Xd.flat, Yd.flat), self.data.flat,
-                                              (x,y), method="linear")
+            if HAS_SCIPY:
+                Xd, Yd = self.center_coords()
+                return griddata((Xd.flat, Yd.flat), self.data.flat,
+                                (x,y), method="linear")
+            else:
+                raise NotImplementedError("linear resampling currently requires"
+                                          " scipy")
         else:
             raise ValueError("method \"{0}\" not understood".format(method))
 
+    def get_extents(self):
+        hdr = self.get_hdr()
+        x1 = hdr["xllcorner"] + hdr["nx"] * hdr["dx"]
+        y1 = hdr["yllcorner"] + hdr["ny"] * hdr["dy"]
+        return (hdr["xllcorner"], x1, hdr["yllcorner"], y1)
+
     def get_profile(self, segments, resolution=10.0):
         """ Sample along a line defined as `segments`. Does not interpolate.
 
@@ -456,10 +466,10 @@ class RegularGrid(Grid):
 class WarpedGrid(Grid):
     """ Warped Grid class. A WarpedGrid contains a fixed number of rows
     and columns and a scalar or vector field defined on the z-axis. Grid
-    spacing is not necessarily regular.
+    spacing is not necessarily constant.
     """
 
-    def __init__(self, hdr=None, X=None, Y=None, Z=None):
+    def __init__(self, X, Y, Z, hdr=None):
         """
         Parameters:
         -----------
@@ -472,12 +482,11 @@ class WarpedGrid(Grid):
             - nbands (int)
         """
 
-        if True in (a is None for a in (X,Y,Z)):
-            raise GridError('Either all or none of (`X`, `Y`, `Z`) must be '
-                            'provided')
+        if any(a is None for a in (X, Y, Z)):
+            raise GridError('All of (X, Y, Z) must be provided')
 
         if not (X.shape == Y.shape == Z.shape[:2]):
-            raise GridError('All of (`X`, `Y`, `Z`) must share the same '
+            raise GridError('All of (X, Y, Z) must share the same '
                             'two-dimensional size')
 
         if hdr is None:
