@@ -1,10 +1,9 @@
 """
-Geographical measurement and simple analysis module for Python 2.X.X. Provides
+Geographical measurement and simple analysis module for Python. Provides
 Point, Multipoint, Line, and Polygon classes, with methods for simple
 measurements such as distance, area, and bearing.
-
-Written by Nat Wilson (njwilson23@gmail.com)
 """
+from __future__ import division
 
 import math
 import sys
@@ -15,20 +14,14 @@ from . import geojson
 from . import xyfile
 from . import _shpfuncs
 from .. import crs as kcrs
-
-from collections import deque
 from .metadata import Metadata
+from . import _vectorgeo
 
 try:
     from . import _cvectorgeo as _vecgeo
 except ImportError:
     sys.stderr.write("falling back on slow _vectorgeo")
-    from . import _vectorgeo as _vecgeo
-
-try:
-    import shapely.geometry as geometry
-except ImportError:
-    pass
+    _vecgeo = _vectorgeo
 
 try:
     import pyproj
@@ -38,8 +31,7 @@ except ImportError:
     PYPROJ = False
 
 class Geometry(object):
-    """ This is the abstract base class for all geometry types, i.e. Point,
-    Multipoints and subclasses thereof. """
+    """ This is the abstract base class for all geometry types """
     _geotype = None
 
     def __init__(self, crs=kcrs.CARTESIAN):
@@ -79,21 +71,28 @@ class Geometry(object):
 
 
 class Point(Geometry):
-    """ This defines the point class, from which x,y[,z] points can be
-    constructed. """
+    """ Point object instantiated with:
+
+    *coords*        2-tuple or 3-tuple
+    *data*          List, dictionary, or Metadata object for point-specific
+                    data [default None]
+    *properties*    Dictionary of geometry specific data [default None]
+    *crs*           Coordinate reference system instance [default CARTESIAN]
+    """
     _geotype = "Point"
 
-    def __init__(self, coords, data=None, properties=None, **kwargs):
-        if properties is None:
-            properties = {}
+    def __init__(self, coords, data=None, properties=None, copy_metadata=True,
+                 **kwargs):
         if not hasattr(coords, "__iter__"):
             raise ValueError("Point coordinates must be a sequence")
         super(Point, self).__init__(**kwargs)
         self.vertex = coords
         self._setxyz()
-        self.data = Metadata(data, singleton=True)
+        self.data = Metadata(data, singleton=True, copydata=copy_metadata)
         if hasattr(properties, "keys"):
             self.properties = properties
+        else:
+            properties = {}
         return
 
     def __getitem__(self, idx):
@@ -103,23 +102,28 @@ class Point(Geometry):
         return 'Point(' + str(self.vertex) + ')'
 
     def __eq__(self, other):
-        if hasattr(other, "vertex"):
+        try:
             return (tuple(self.vertex) == tuple(other.vertex)) and \
                    (self.data == other.data) and \
                    (self.properties == other.properties)
-        else:
+        except AttributeError:
             return False
 
+    @property
+    def __geo_interface__(self):
+        return {"type" : "Point", "coordinates" : self.vertex}
+
     def _setxyz(self):
         """ Create convenience *x*, *y* (, *z*) attributes from vertices. """
-        self.x = self.vertex[0]
-        self.y = self.vertex[1]
-        try:
-            self.z = self.vertex[2]
-            self.rank = 3
-        except IndexError:
-            self.z = None
+        vertex = self.vertex
+        self.x = vertex[0]
+        self.y = vertex[1]
+        if len(vertex) == 2:
             self.rank = 2
+            self.z = None
+        else:
+            self.rank = 3
+            self.z = vertex[2]
         return
 
     def isat(self, other, tol=0.001):
@@ -191,8 +195,8 @@ class Point(Geometry):
             direction (float): horizontal walk direction in radians
         """
         if self._crs == kcrs.CARTESIAN:
-            dx = distance * math.cos(direction)
-            dy = distance * math.sin(direction)
+            dx = distance * math.sin(direction)
+            dy = distance * math.cos(direction)
 
             if self.rank == 2:
                 return Point((self.x+dx, self.y+dy),
@@ -281,39 +285,23 @@ class Point(Geometry):
                 fobj.close()
         return writer
 
-    def to_shapely(self):
-        """ Returns a Shapely Point instance. """
-        try:
-            return geometry.Point(self.x, self.y, self.z)
-        except NameError:
-            raise ImportError('Shapely module did not import\n')
-
 
 class MultipointBase(Geometry):
     """ Point cloud with associated attributes. This is a base class for the
     polyline and polygon classes. """
     _geotype = "MultipointBase"
 
-    def __init__(self, vertices, data=None, properties=None, **kwargs):
-        """ Create a feature with multiple vertices.
-
-        vertices : a list of tuples containing point coordinates.
-
-        data : is either `None` a list of point attributes, or a dictionary of
-        point attributes. If `data` is not `None`, then it (or its values) must
-        match `vertices` in length.
-        """
+    def __init__(self, vertices, data=None, properties=None, copy_metadata=True,
+                 **kwargs):
         super(MultipointBase, self).__init__(**kwargs)
         vertices = list(vertices)
-        if properties is None:
-            properties = {}
         if len(vertices) > 0:
             self.rank = len(vertices[0])
 
-            if self.rank > 3 or self.rank < 2:
-                raise GInitError('Input must be doubles or triples\n')
-            elif False in [self.rank == len(i) for i in vertices]:
-                raise GInitError('Input must have consistent rank\n')
+            if not 2 <= self.rank <= 3:
+                raise GInitError("Input must be doubles or triples")
+            elif not all(self.rank == len(i) for i in vertices):
+                raise GInitError("Input must have consistent rank")
             else:
                 self.vertices = [tuple(i) for i in vertices]
 
@@ -323,8 +311,12 @@ class MultipointBase(Geometry):
 
         if hasattr(properties, "keys"):
             self.properties = properties
+        elif properties is None:
+            self.properties = {}
+        else:
+            raise GInitError("value provided as 'properties' must be a hash")
 
-        self.data = Metadata(data)
+        self.data = Metadata(data, copydata=copy_metadata)
         return
 
     def __repr__(self):
@@ -333,15 +325,7 @@ class MultipointBase(Geometry):
         else:
             ppverts = str(self.vertices[:2])[:-1] + "..." + str(self.vertices[-2:])[1:]
         return '{typ}({verts})>'.format(
-                typ=str(type(self))[:-1], verts=ppverts, prop=self.properties)
-
-    def __str__(self):
-        if len(self) < 5:
-            ppverts = str(self.vertices)
-        else:
-            ppverts = str(self.vertices[:2])[:-1] + "..." + str(self.vertices[-2:])[1:]
-        return '{typ}\n{verts}\nProperties:{prop}'.format(
-                typ=type(self), verts=ppverts, prop=self.properties)
+                typ=str(type(self))[:-1], verts=ppverts)
 
     def __len__(self):
         return len(self.vertices)
@@ -349,10 +333,12 @@ class MultipointBase(Geometry):
     def __getitem__(self, key):
         if isinstance(key, (int, np.int64)):
             return Point(self.vertices[key], data=self.data[key],
-                         properties=self.properties, crs=self._crs)
+                         properties=self.properties, crs=self._crs,
+                         copy_metadata=False)
         elif isinstance(key, slice):
             return type(self)(self.vertices[key], data=self.data[key],
-                              properties=self.properties, crs=self._crs)
+                              properties=self.properties, crs=self._crs,
+                              copy_metadata=False)
         else:
             raise GGeoError('Index must be an integer or a slice object')
 
@@ -386,27 +372,27 @@ class MultipointBase(Geometry):
         return hasattr(other, "_geotype") and \
                (self._geotype == other._geotype) and \
                (len(self) == len(other)) and \
-               (False not in (a==b for a,b in zip(self, other)))
+               all(a==b for a,b in zip(self, other))
 
     def _bbox_overlap(self, other):
         """ Return whether bounding boxes between self and another geometry
         overlap.
         """
-        reg0 = self.get_bbox()
-        reg1 = other.get_bbox()
-        return (reg0[0] < reg1[1] and reg0[1] > reg1[0] and
-                reg0[2] < reg1[3] and reg0[3] > reg1[2])
+        reg0 = self.bbox
+        reg1 = other.bbox
+        return (reg0[0] < reg1[2] and reg0[2] > reg1[0] and
+                reg0[1] < reg1[3] and reg0[3] > reg1[1])
 
-    def get_bbox(self):
+    @property
+    def bbox(self):
         """ Return the extents of a bounding box as
-            (xmin, ymax, ymin, ymax, [zmin, zmin]).
+            (xmin, ymin, xmax, ymax)
         """
         if self.rank == 2:
             x, y = self.get_coordinate_lists()
-            bbox = (min(x), max(x), min(y), max(y))
         elif self.rank == 3:
             x, y, z = self.get_coordinate_lists()
-            bbox = (min(x), max(x), min(y), max(y), min(z), max(z))
+        bbox = (min(x), min(y), max(x), max(y))
         return bbox
 
     def print_vertices(self):
@@ -471,7 +457,8 @@ class MultipointBase(Geometry):
         """ Return a subset defined by index in *idxs*. """
         vertices = [self.vertices[i] for i in idxs]
         data = self.data.sub(idxs)
-        subset = type(self)(vertices, data=data, properties=self.properties, crs=self._crs)
+        subset = type(self)(vertices, data=data, properties=self.properties,
+                            crs=self._crs, copy_metadata=False)
         return subset
 
     def flat_distances_to(self, pt):
@@ -492,8 +479,8 @@ class MultipointBase(Geometry):
         return np.array(d)
 
     def nearest_point_to(self, pt):
-        """ Returns the internal point that is nearest to pt (Point class).
-        If two points are equidistant, only one will be returned.
+        """ Returns the internal Point that is nearest to *pt*. If two points
+        are equidistant, only one will be returned.
         """
         distances = self.distances_to(pt)
         idx = np.argmin(distances)
@@ -527,6 +514,13 @@ class MultipointBase(Geometry):
     #    D = map(dist, (p[0] for p in P), (p[1] for p in P))
     #    return P[D.index(max(D))]
 
+    def any_within_poly(self, poly):
+        """ Return whether any vertices are inside *poly* """
+        for pt in self:
+            if poly.contains(pt):
+                return True
+        return False
+
     def to_xyfile(self, fnm, fields=None, delimiter=' ', header=None):
         """ Write data to a delimited ASCII table.
 
@@ -594,57 +588,67 @@ class MultipointBase(Geometry):
         elif self.rank == 3:
             _shpfuncs.write_multipoint3(self, fstem)
         else:
-            raise IOError("rank must be 2 or 3 to write as a shapefile\n")
+            raise IOError("Rank must be 2 or 3 to write as a shapefile")
         return
 
+
 class Multipoint(MultipointBase):
     """ Point cloud with associated attributes. This is a base class for the
-    polyline and polygon classes. """
+    polyline and polygon classes.
+
+    *coords*        List of 2-tuples or 3-tuples
+    *data*          List, dictionary, or Metadata object for point-specific
+                    data [default None]
+    *properties*    Dictionary of geometry specific data [default None]
+    *crs*           Coordinate reference system instance [default CARTESIAN]
+    """
     _geotype = "Multipoint"
 
-    def __init__(self, vertices, data=None, properties=None, **kwargs):
-        """ Create a feature with multiple vertices.
-
-        vertices : a list of tuples containing point coordinates.
-
-        data : is either `None` a list of point attributes, or a dictionary of
-        point attributes. If `data` is not `None`, then it (or its values) must
-        match `vertices` in length.
-        """
-        if False not in (hasattr(v, "_geotype") and \
-                         v._geotype == "Point" and \
-                         v.rank == vertices[0].rank for v in vertices):
+    def __init__(self, vertices, data=None, properties=None, copy_metadata=True,
+                 **kwargs):
+        def ispoint(a):
+            return hasattr(a, "_geotype") and a._geotype == "Point"
+        if len(vertices) != 0 and all(ispoint(a) for a in vertices):
             points = vertices
             crs = points[0]._crs
-            if False in (pt._crs == crs for pt in points):
+            rank = points[0].rank
+            if len(points) != 1 and not all(pt._crs == crs for pt in points[1:]):
                 raise CRSError("All points must share the same CRS")
+            elif not all(rank == pt.rank for pt in points[1:]):
+                raise GInitError("Input must have consistent rank")
 
-            keys = list(points[0].properties.keys())
-            for pt in points[1:]:
-                for key in keys:
-                    if key not in pt.properties:
-                        keys.pop(keys.index(key))
+            keys = list(points[0].data.keys())
+            if len(points) != 1:
+                for pt in points[1:]:
+                    for key in keys:
+                        if key not in pt.data:
+                            keys.pop(keys.index(key))
 
             ptdata = {}
             for key in keys:
-                ptdata[key] = [pt.properties[key] for pt in points]
+                ptdata[key] = [pt.data[key] for pt in points]
 
             if data is not None:
                 ptdata.update(data)
-            self.data = Metadata(ptdata)
+
+            self.data = Metadata(ptdata, copydata=copy_metadata)
             self.vertices = [pt.vertex for pt in points]
-            self.rank = vertices[0].rank
             self.properties = properties
+            self.rank = rank
             self._crs = crs
 
         else:
 
             super(Multipoint, self).__init__(vertices,
-                                             data=data, 
+                                             data=data,
                                              properties=properties,
                                              **kwargs)
         return
 
+    @property
+    def __geo_interface__(self):
+        return {"type" : "MultiPoint", "bbox" : self.bbox, "coordinates" : self.vertices}
+
     def within_radius(self, pt, radius):
         """ Return Multipoint of subset that is within *radius* of *pt*.
         """
@@ -653,7 +657,7 @@ class Multipoint(MultipointBase):
         return self._subset(indices)
 
     def within_bbox(self, bbox):
-        """ Return Multipoint subset that is within a square boundaing box
+        """ Return Multipoint subset that is within a square bounding box
         given by (xmin, xmax, ymin, ymax). """
         filtbbox = lambda pt: (bbox[0] <= pt.vertex[0] <= bbox[1]) and \
                               (bbox[2] <= pt.vertex[1] <= bbox[3])
@@ -671,30 +675,40 @@ class ConnectedMultipoint(MultipointBase):
         return sum(distances)
 
     def segments(self):
-        """ Returns an iterator of adjacent line segments. """
+        """ Returns an generator of adjacent line segments. """
         return (self._subset((i,i+1)) for i in range(len(self)-1))
 
     def segment_tuples(self):
-        """ Returns an iterator of adjacent line segments as coordinate tuples. """
+        """ Returns an generator of adjacent line segments as coordinate tuples. """
         return ((self.vertices[i], self.vertices[i+1])
                 for i in range(len(self.vertices)-1))
 
     def intersects(self, other):
         """ Return whether an intersection exists with another geometry. """
-        interxbool = (_vecgeo.intersects(a[0][0], a[1][0], b[0][0], b[1][0],
-                                         a[0][1], a[1][1], b[0][1], b[1][1])
+        interxbool = (np.nan in _vecgeo.intersection(a[0][0], a[1][0], b[0][0], b[1][0],
+                                                     a[0][1], a[1][1], b[0][1], b[1][1])
                     for a in self.segments() for b in other.segments())
-        if self._bbox_overlap(other) and (True in interxbool):
+        if self._bbox_overlap(other) and (True not in interxbool):
             return True
         else:
             return False
 
-    def intersections(self, other):
-        """ Return the intersections with another geometry. """
-        interx = (_vecgeo.intersections(a[0][0], a[1][0], b[0][0], b[1][0],
-                                        a[0][1], a[1][1], b[0][1], b[1][1])
-                    for a in self.segments() for b in other.segments())
-        return list(filter(lambda a: np.nan not in a, interx))
+    def intersections(self, other, keep_duplicates=False):
+        """ Return the intersections with another geometry as a Multipoint. """
+        interx = (_vecgeo.intersection(a[0][0], a[1][0], b[0][0], b[1][0],
+                                          a[0][1], a[1][1], b[0][1], b[1][1])
+                     for a in self.segments() for b in other.segments())
+        if not keep_duplicates:
+            interx = set(interx)
+        interx_points = []
+        for vertex in interx:
+            if np.nan not in vertex:
+                interx_points.append(Point(vertex, properties=self.properties,
+                                           crs=self._crs))
+        nandata = [np.nan for _ in interx_points]
+        keys = [k for k in set(list(self.data.keys()) + list(other.data.keys()))]
+        d = Metadata(dict((key,nandata) for key in keys))
+        return Multipoint(interx_points, data=d)
 
     def shortest_distance_to(self, pt):
         """ Return the shortest distance from any position on the Multipoint
@@ -707,10 +721,11 @@ class ConnectedMultipoint(MultipointBase):
         else:
             func = lambda *args: _vecgeo.pt_nearest_proj(geod, *args, tol=0.01)
 
+        segments = list(self.segments())
         point_dist = map(func,
-                         [pt.vertex for seg in self.segments()],
-                         [seg[0].vertex for seg in self.segments()],
-                         [seg[1].vertex for seg in self.segments()])
+                         (pt.vertex for _ in segments),
+                         (seg[0].vertex for seg in segments),
+                         (seg[1].vertex for seg in segments))
         distances = [i[1] for i in point_dist]
         return min(distances)
 
@@ -725,27 +740,35 @@ class ConnectedMultipoint(MultipointBase):
         else:
             func = lambda *args: _vecgeo.pt_nearest_proj(geod, *args, tol=0.01)
 
+        segments = list(self.segments())
         point_dist = list(map(func,
-                              [pt.vertex for seg in self.segments()],
-                              [seg[0].vertex for seg in self.segments()],
-                              [seg[1].vertex for seg in self.segments()]))
+                              (pt.vertex for _ in segments),
+                              (seg[0].vertex for seg in segments),
+                              (seg[1].vertex for seg in segments)))
         distances = [i[1] for i in point_dist]
         return Point(point_dist[distances.index(min(distances))][0],
                      properties=self.properties, crs=self._crs)
 
     def within_distance(self, pt, distance):
         """ Test whether a point is within *distance* of a ConnectedMultipoint. """
-        # Avoid calculating more minimum distances than necessary
-        return True in (distance >= seg.shortest_distance_to(pt)
-                        for seg in self.segments())
+        return all(distance >= seg.shortest_distance_to(pt) for seg in self.segments())
+
 
 class Line(ConnectedMultipoint):
-    """ This defines the polyline class, from which geographic line
-    objects can be constructed. Line objects consist of joined,
-    georeferenced line segments.
+    """ Line composed of connected vertices.
+
+    *coords*        List of 2-tuples or 3-tuples
+    *data*          List, dictionary, or Metadata object for point-specific
+                    data [default None]
+    *properties*    Dictionary of geometry specific data [default None]
+    *crs*           Coordinate reference system instance [default CARTESIAN]
     """
     _geotype = "Line"
 
+    @property
+    def __geo_interface__(self):
+        return {"type" : "LineString", "bbox" : self.bbox, "coordinates" : self.vertices}
+
     def add_vertex(self, vertex):
         """ Add a vertex to self.vertices. """
         if isinstance(vertex, Point):
@@ -790,9 +813,48 @@ class Line(ConnectedMultipoint):
             pta = ptb
         return d
 
+    def subsection(self, n):
+        """ Return *n* equally spaced Point instances along line. """
+        segments = self.segments()
+        Ltotal = self.cumlength()[-1]
+        step = Ltotal / float(n-1)
+        step_remaining = step
+
+        points = [self[0]]
+        x = 0.0
+        pos = self[0]
+        seg = next(segments)
+        seg_remaining = seg.displacement()
+
+        while x < Ltotal-1e-8:
+            direction = seg[0].azimuth(seg[1])
+
+            if step_remaining <= seg_remaining:
+                pos = pos.walk(step_remaining, direction)
+                x += step_remaining
+                seg_remaining -= step_remaining
+                step_remaining = step
+                points.append(pos)
+                seg.vertices[0] = pos.vertex
+
+            else:
+                pos = seg[1]
+                x += seg_remaining
+                step_remaining -= seg_remaining
+
+                seg = next(segments, seg)
+                seg_remaining = seg.displacement()
+                # except StopIteration as e:
+                #     if abs(Ltotal-x) > 1e-8:       # tolerance for endpoint
+                #         raise e
+
+        if len(points) == n-1:
+            points.append(seg[-1])
+        return points
+
     def displacement(self):
         """ Returns the distance between the first and last vertex. """
-        return Point(self.vertices[0]).distance(Point(self.vertices[-1]))
+        return self[0].distance(self[-1])
 
     #def direction(self):
     #    """ Returns a vector of the azimuth along a line at each point,
@@ -820,16 +882,6 @@ class Line(ConnectedMultipoint):
         """ Returns a polygon. """
         return Polygon(self.vertices)
 
-    def to_shapely(self):
-        """ Returns a Shapely LineString instance. """
-        try:
-            if self.rank == 2:
-                return geometry.LineString([(v[0], v[1]) for v in self.vertices])
-            elif self.rank == 3:
-                return geometry.LineString([(v[0], v[1], v[2]) for v in self.vertices])
-        except NameError:
-            raise GuppyError('Shapely module not available\n')
-
     def to_shapefile(self, fstem):
         """ Save line to a shapefile """
         if self.rank == 2:
@@ -837,13 +889,19 @@ class Line(ConnectedMultipoint):
         elif self.rank == 3:
             _shpfuncs.write_line3(self, fstem)
         else:
-            raise IOError("rank must be 2 or 3 to write as a shapefile\n")
+            raise IOError("rank must be 2 or 3 to write as a shapefile")
         return
 
+
 class Polygon(ConnectedMultipoint):
-    """ This defines the polygon class, from which geographic
-    polygons objects can be created. Polygon objects consist of
-    point nodes enclosing an area.
+    """ Polygon, composed of a closed sequence of vertices.
+
+    *coords*        List of 2-tuples or 3-tuples
+    *data*          List, dictionary, or Metadata object for point-specific
+                    data [default None]
+    *properties*    Dictionary of geometry specific data [default None]
+    *subs*          List of sub-polygons [default None]
+    *crs*           Coordinate reference system instance [default CARTESIAN]
     """
     _geotype = "Polygon"
     subs = []
@@ -859,10 +917,22 @@ class Polygon(ConnectedMultipoint):
         if isinstance(key, slice):
             ind = key.indices(len(self))
             if len(self) != ((ind[1] - ind[0]) // ind[2]):
-                return Line(self.vertices[key], data=self.data[key], 
+                return Line(self.vertices[key], data=self.data[key],
                             properties=self.properties, crs=self._crs)
         return super(Polygon, self).__getitem__(key)
 
+    @property
+    def __geo_interface__(self):
+        return {"type" : "Polygon", "bbox" : self.bbox, "coordinates" : self.vertices}
+
+    def _subset(self, idxs):
+        """ Return a subset defined by index in *idxs*. """
+        vertices = [self.vertices[i] for i in idxs]
+        data = self.data.sub(idxs)
+        subset = Line(vertices, data=data, properties=self.properties,
+                      crs=self._crs, copy_metadata=False)
+        return subset
+
     def perimeter(self):
         """ Return the perimeter of the polygon. If there are sub-polygons,
         their perimeters are added recursively. """
@@ -879,48 +949,22 @@ class Polygon(ConnectedMultipoint):
 
     def contains(self, pt):
         """ Returns True if pt is inside or on the boundary of the
-        polygon, and False otherwise.
+        polygon, and False otherwise. Uses a crossing number scheme.
         """
-        def possible(pt, v1, v2):
-            """ Quickly assess potential for an intersection with an x+
-            pointing ray. """
-            x = pt.vertex[0]
-            y = pt.vertex[1]
-            if ( ((y > v1[1]) is not (y > v2[1]))
-            and ((x < v1[0]) or (x < v2[0])) ):
-                return True
-            else:
-                return False
+        nintx = 0
+        x, y = pt[0], pt[1]
+        for seg in self.segments():
+            (a, b) = seg
+            if _vecgeo.intersects_cn(x, y, a[0], b[0], a[1], b[1]):
+                nintx += 1
 
-        # Find how many boundaries a ray pointing out from point crosses
-        bool2int = lambda tf: (tf and 1 or 0)
-        rvertices = deque(self.vertices)
-        rvertices.rotate(1)
-        segments = [(v1, v2) for v1, v2 in zip(self.vertices, rvertices)
-                    if possible(pt, v1, v2)]
-
-        n_intersect = sum([bool2int(
-            isinstance(ray_intersection(pt.vertex, seg[0], seg[1]), tuple))
-            for seg in segments])
-
-        if n_intersect % 2 == 1:    # If odd, point is inside so check subpolys
-            if True not in (p.contains(pt) for p in self.subs):
-                return True
-        return False                # Point was outside or was in a subpoly
+        # If odd, point is inside so check subpolys
+        return nintx % 2 == 1 and not any(p.contains(pt) for p in self.subs)
 
     def to_polyline(self):
         """ Returns a self-closing polyline. Discards sub-polygons. """
         return Line(self.vertices)
 
-    def to_shapely(self):
-        """ Returns a Shapely Polygon instance. """
-        try:
-            shp = geometry.Polygon(self.vertices,
-                                   interiors=[p.vertices for p in self.subs])
-        except NameError:
-            raise ImportError('Shapely module did not import\n')
-        return shp
-
     def to_shapefile(self, fstem):
         """ Save line to a shapefile """
         if self.rank == 2:
@@ -928,7 +972,7 @@ class Polygon(ConnectedMultipoint):
         elif self.rank == 3:
             _shpfuncs.write_poly3(self, fstem)
         else:
-            raise IOError("rank must be 2 or 3 to write as a shapefile\n")
+            raise IOError("rank must be 2 or 3 to write as a shapefile")
         return
 
 
@@ -957,11 +1001,13 @@ class GGeoError(GuppyError):
     def __init__(self, message=''):
         self.message = message
 
+
 class CRSError(GuppyError):
     """ Exception to raise for invalid geodetic operations. """
     def __init__(self, message=''):
         self.message = message
 
+
 def ray_intersection(pt, endpt1, endpt2, direction=0.0):
     """ Determines whether a ray intersects a line segment. If yes,
     returns the point of intersection. If no, return None. Input
@@ -1064,7 +1110,7 @@ def points_to_multipoint(points):
     as Multipoint data. All points must use the same CRS.
     """
     crs = points[0]._crs
-    if False in (pt._crs == crs for pt in points):
+    if not all(pt._crs == crs for pt in points):
         raise CRSError("All points must share the same CRS")
 
     keys = list(points[0].properties.keys())
